from time import sleep
from queue import Queue
from threading import Thread


class MessageQueueHandler:
    keys = ['message.queue']

    def __init__(self, wpm):
        self.connector = None
        self._wpm, self._cps = None, None
        self.wpm = wpm
        self._queue = Queue()
        self._thread = Thread(target=self._dispatch_messages)

    @property
    def wpm(self):
        return self._wpm

    @wpm.setter
    def wpm(self, wpm):
        self._wpm = wpm
        self._cps = 5 * wpm / 60

    def _dispatch_messages(self):
        for message in iter(self._queue.get, None):
            sleep(len(message['message'])/self._cps)
            self.connector.send_message(message)

    def handle_event(self, message, _):
        for unpacked in self._generate_messages_from_queue(message):
            self._queue.put(unpacked)

    @staticmethod
    def _generate_messages_from_queue(queue_message):
        last = queue_message['value'][-1]
        for message in queue_message['value']:
            yield {
                'key': 'message.send',
                'typing': message is not last,
                **message
            }

    def start(self):
        self._thread.start()

    def cleanup(self):
        self._queue.queue.clear()
        self._queue.put(None)
        self._thread.join()
